@@ -58,6 +58,7 @@ class GetData:
                 ACCESS_TOKEN = broker_config["ACCESS_TOKEN"]
                 port = broker_config["PORT"]
                 self.api = v20.Context(hostname=API, token=ACCESS_TOKEN, port=port)
+                self.ACCOUNT_ID = broker_config["ACCOUNT_ID"]
             
             elif broker_config['data_source'] == 'IB':
                 host = broker_config['host']
@@ -238,7 +239,8 @@ class GetData:
     
     
     def _oanda_quote_data(self, data: pd.DataFrame, pair: str, granularity: str, 
-                         start_time: datetime, end_time: datetime):
+                         start_time: datetime, end_time: datetime, 
+                         count: int = None):
         """Function to retrieve price conversion data.
         """
         base_currency = pair[:3]
@@ -250,25 +252,40 @@ class GetData:
             
         else:
             if self.home_currency == base_currency:
-                # Invert data to get quote data
-                quote_data = 1/data
+                # Disturb the data by machine precision to prompt HCF calculation
+                quote_data = (1 + 1e-15) * data
                 
             else:
                 # Try download quote data
-                try:
-                    conversion_pair = self.home_currency + "_" + quote_currency
-                    if conversion_pair != pair:
-                        # Do not re-download the same data
-                        quote_data = self.oanda(instrument  = conversion_pair,
-                                                granularity = granularity,
-                                                start_time  = start_time,
-                                                end_time    = end_time)
-                    else:
-                        quote_data = data
-                        
-                except:
-                    # Download failed, revert to original data
+                conversion_pair = self.home_currency + "_" + quote_currency
+                if conversion_pair == pair:
+                    # Same instrument
                     quote_data = data
+                    
+                else:
+                    # Different instrument, fetch data
+                    try:
+                        # Directly
+                        quote_data = self.oanda(instrument=conversion_pair,
+                                                granularity=granularity,
+                                                start_time=start_time,
+                                                end_time=end_time,
+                                                count=count)
+                    except:
+                        # Failed
+                        try:
+                            # Invert conversion pair
+                            conversion_pair = quote_currency + "_" + self.home_currency
+                            inverse_quote_data = self.oanda(instrument=conversion_pair,
+                                                    granularity=granularity,
+                                                    start_time=start_time,
+                                                    end_time=end_time,
+                                                    count=count)
+                            quote_data = 1/inverse_quote_data[['Open', 'High', 'Low', 'Close']]
+                            
+                        except:
+                            # Failed, just used original data
+                            quote_data = data
         
         return quote_data
     
@@ -283,27 +300,41 @@ class GetData:
     def response_to_df(self, response):
         """Function to convert api response into a pandas dataframe.
         """
-        candles = response.body["candles"]
+        try:
+            candles = response.body["candles"]
+        except KeyError:
+            raise Exception("Error dowloading data - please check instrument"+\
+                            " format and try again.")
+            
         times = []
-        close_price, high_price, low_price, open_price = [], [], [], []
+        close_price, high_price, low_price, open_price, volume = [],[],[],[],[]
         
-        for candle in candles:
-            if self.allow_dancing_bears:
+        if self.allow_dancing_bears:
+            # Allow all candles
+            for candle in candles:
                 times.append(candle.time)
                 close_price.append(float(candle.mid.c))
                 high_price.append(float(candle.mid.h))
                 low_price.append(float(candle.mid.l))
                 open_price.append(float(candle.mid.o))
+                volume.append(float(candle.volume))
                 
-            else:
+        else:
+            # Only allow complete candles
+            for candle in candles:
                 if candle.complete:
                     times.append(candle.time)
                     close_price.append(float(candle.mid.c))
                     high_price.append(float(candle.mid.h))
                     low_price.append(float(candle.mid.l))
                     open_price.append(float(candle.mid.o))
+                    volume.append(float(candle.volume))
         
-        dataframe = pd.DataFrame({"Open": open_price, "High": high_price, "Low": low_price, "Close": close_price})
+        dataframe = pd.DataFrame({"Open": open_price, 
+                                  "High": high_price, 
+                                  "Low": low_price, 
+                                  "Close": close_price,
+                                  "Volume": volume})
         dataframe.index = pd.to_datetime(times)
         dataframe.drop_duplicates(inplace=True)
         
@@ -395,8 +426,9 @@ class GetData:
                            end      = end_time,
                            interval = granularity)
         
-        # Add UTC timezone
-        data.index = data.index.tz_localize(timezone.utc)
+        if data.index.tzinfo is None:
+            # Data is naive, add UTC timezone
+            data.index = data.index.tz_localize(timezone.utc)
         
         return data
     
@@ -406,7 +438,8 @@ class GetData:
         
     
     def _yahoo_quote_data(self, data: pd.DataFrame, pair: str, interval: str,
-                         from_date: datetime, to_date: datetime):
+                         from_date: datetime, to_date: datetime, 
+                         count: int = None):
         """Returns nominal price data - quote conversion not supported for 
         Yahoo finance API.
         """
@@ -543,8 +576,8 @@ class GetData:
                 positiveHCF = 1
             else:
                 # Quote data
-                negativeHCF = quote_price
-                positiveHCF = quote_price
+                negativeHCF = 1/quote_price
+                positiveHCF = 1/quote_price
                 
         else:
             # No quote price provided
