@@ -70,6 +70,8 @@ class Broker:
             response = self._place_limit_order(order)
         elif order.order_type == 'close':
             response = self._close_position(order.instrument)
+        elif order.order_type == 'modify':
+            response = self._modify_trade(order)
         else:
             print("Order type not recognised.")
         
@@ -138,25 +140,26 @@ class Broker:
         for trade in oanda_open_trades:
             new_trade = {}
             related_orders = []
-            new_trade['order_ID']           = trade.id
-            new_trade['order_stop_price']   = trade.price
-            new_trade['order_limit_price']  = trade.price
-            new_trade['direction']          = np.sign(trade.currentUnits)
-            new_trade['order_time']         = trade.openTime
-            new_trade['instrument']         = trade.instrument
-            new_trade['size']               = abs(trade.currentUnits)
-            new_trade['order_price']        = trade.price
-            new_trade['entry_price']        = trade.price
+            new_trade['instrument'] = trade.instrument
+            new_trade['time_filled'] = trade.openTime
+            new_trade['fill_price'] = trade.price
+            new_trade['size'] = abs(trade.currentUnits)
+            new_trade['id'] = trade.id
+            new_trade['direction'] = np.sign(trade.currentUnits)
+            new_trade['margin_required'] = trade.marginUsed
+            new_trade['unrealised_PL'] = trade.unrealizedPL
+            new_trade['fees'] = trade.financing
+            new_trade['status'] = trade.state.lower()
             
             # Check for take profit
             if trade.takeProfitOrder is not None:
-                new_trade['take_profit']    = trade.takeProfitOrder.price
+                new_trade['take_profit'] = trade.takeProfitOrder.price
                 related_orders.append(trade.takeProfitOrder.id)
             
             # Check for stop loss
             if trade.stopLossOrder is not None:
-                new_trade['stop_loss']    = trade.stopLossOrder.price
-                new_trade['stop_type']    = 'limit'
+                new_trade['stop_loss'] = trade.stopLossOrder.price
+                new_trade['stop_type'] = 'limit'
                 related_orders.append(trade.stopLossOrder.id)
             
             if related_orders is not None:
@@ -172,6 +175,7 @@ class Broker:
     
     def get_trade_details(self, trade_ID: int):
         """Returns the details of the trade specified by trade_ID.
+        WARNING: THIS METHOD HAS BEEN MADE REDUNDANT BY 'get_trades' METHOD.
         """
         
         response = self.api.trade.list(accountID=self.ACCOUNT_ID, ids=int(trade_ID))
@@ -335,11 +339,11 @@ class Broker:
         
         self._check_connection()
         
-        response        = self.api.instrument.candles(instrument,
-                                                      granularity = interval,
-                                                      fromTime = from_time,
-                                                      toTime = to_time
-                                                      )
+        response = self.api.instrument.candles(instrument,
+                                               granularity = interval,
+                                               fromTime = from_time,
+                                               toTime = to_time
+                                               )
         
         data = self.utils.response_to_df(response)
         
@@ -452,15 +456,6 @@ class Broker:
         return response.body['instruments'][0].pipLocation
     
     
-    def get_trade_unit_precision(self, instrument: str):
-        """Returns the trade unit precision for the requested instrument.
-        """
-        # TODO - implement checking using this method
-        response = self.api.account.instruments(self.ACCOUNT_ID, 
-                                                instruments=instrument)
-        return response.body['instruments'][0].tradeUnitsPrecision
-    
-    
     def _check_connection(self) -> None:
         """Connects to Oanda v20 REST API. An initial call is performed to check
         for a timeout error.
@@ -510,7 +505,7 @@ class Broker:
         """Places market order.
         """
         self._check_connection()
-        stop_loss_details = self._get_stop_loss_details(order)
+        stop_loss_order = self._get_stop_loss_order(order)
         take_profit_details = self._get_take_profit_details(order)
         
         # Check position size
@@ -521,7 +516,8 @@ class Broker:
                                          instrument = order.instrument,
                                          units = order.direction * size,
                                          takeProfitOnFill = take_profit_details,
-                                         stopLossOnFill = stop_loss_details,)
+                                         **stop_loss_order)
+        
         return response
     
     
@@ -529,38 +525,101 @@ class Broker:
         """Places MarketIfTouchedOrder with Oanda.
         https://developer.oanda.com/rest-live-v20/order-df/
         """
+        # TODO - this submits market if touched, not stopOrder type
         self._check_connection()
         
-        stop_loss_details = self._get_stop_loss_details(order)
+        stop_loss_order = self._get_stop_loss_order(order)
         take_profit_details = self._get_take_profit_details(order)
         
         # Check and correct order stop price
         price = self._check_precision(order.instrument, 
                                       order.order_stop_price)
-        
         trigger_condition = order.trigger_price
+        size = self.check_trade_size(order.instrument, order.size)
         
         # Need to test cases when no stop/take is provided (as None type)
         # TODO - support other order types (see link above, market is default)
         response = self.api.order.market_if_touched(accountID = self.ACCOUNT_ID,
                                                     instrument = order.instrument,
-                                                    units = order.direction * order.size,
+                                                    units = order.direction * size,
                                                     price = str(price),
                                                     takeProfitOnFill = take_profit_details,
-                                                    stopLossOnFill = stop_loss_details,
-                                                    triggerCondition = trigger_condition)
+                                                    triggerCondition = trigger_condition,
+                                                    **stop_loss_order)
         return response
     
     
     def _place_limit_order(self, order: Order):
-        """(NOT YET IMPLEMENTED) PLaces limit order. 
+        """PLaces a limit order. 
         """
-        raise Exception("Limit orders are not yet implemented for Oanda. "+\
-                        "Please raise an issue on GitHub.")
+        self._check_connection()
         
+        stop_loss_order = self._get_stop_loss_order(order)
+        take_profit_details = self._get_take_profit_details(order)
+        
+        # Check and correct order stop price
+        price = self._check_precision(order.instrument, 
+                                      order.order_limit_price)
+        
+        trigger_condition = order.trigger_price
+        size = self.check_trade_size(order.instrument, order.size)
+        
+        response = self.api.order.limit(accountID = self.ACCOUNT_ID,
+                                        instrument = order.instrument,
+                                        units = order.direction * size,
+                                        price = str(price),
+                                        takeProfitOnFill = take_profit_details,
+                                        triggerCondition = trigger_condition,
+                                        **stop_loss_order)
+        return response
+    
 
-    def _get_stop_loss_details(self, order: Order) -> dict:
-        """Constructs stop loss details dictionary.
+    def _modify_trade(self, order):
+        """Modifies the take profit and/or stop loss of an existing trade.
+
+        Parameters
+        ----------
+        order : TYPE
+            DESCRIPTION.
+        """
+        # Get ID of trade to modify
+        modify_trade_id = order.related_orders
+        trade = self.api.trade.get(accountID=self.ACCOUNT_ID, 
+                                   tradeSpecifier=modify_trade_id).body['trade']
+        
+        if order.take_profit is not None:
+            # Modify trade take-profit
+            tpID = trade.takeProfitOrder.id
+            
+            # Cancel existing TP
+            self.api.order.cancel(self.ACCOUNT_ID, tpID)
+            
+            # Create new TP
+            tp_price = self._check_precision(order.instrument, order.take_profit)
+            new_tp_order = self.api.order.TakeProfitOrder(tradeID=str(modify_trade_id), 
+                                                          price=str(tp_price))
+            response = self.api.order.create(accountID=self.ACCOUNT_ID, 
+                                             order=new_tp_order)
+            self._check_response(response)
+        
+        if order.stop_loss is not None:
+            # Modify trade stop-loss
+            slID = trade.stopLossOrder.id
+            
+            # Cancel existing SL
+            self.api.order.cancel(self.ACCOUNT_ID, slID)
+            
+            # Create new SL
+            sl_price = self._check_precision(order.instrument, order.stop_loss)
+            new_sl_order = self.api.order.StopLossOrder(tradeID=str(modify_trade_id), 
+                                                        price=str(sl_price))
+            response = self.api.order.create(accountID=self.ACCOUNT_ID, 
+                                             order=new_sl_order)
+            self._check_response(response)
+        
+        
+    def _get_stop_loss_order(self, order: Order) -> dict:
+        """Constructs stop loss order dictionary.
         """
         self._check_connection()
         if order.stop_type is not None:
@@ -568,14 +627,41 @@ class Broker:
             
             if order.stop_type == 'trailing':
                 # Trailing stop loss order
-                stop_loss_details = {"price": str(price),
-                                     "type": "TRAILING_STOP_LOSS"}
+                SL_type = "trailingStopLossOnFill"
+                
+                # Calculate stop loss distance
+                if order.stop_distance is None:
+                    # Calculate stop distance from stop loss price provided
+                    if order._working_price is not None:
+                        working_price = order._working_price
+                    else:
+                        if order.order_type == 'market':
+                            # Get current market price
+                            last = self._get_price(order.instrument)
+                            working_price = last.closeoutBid if order.direction < 0 else last.closeoutAsk
+                        elif order.order_type in ['limit', 'stop-limit']:
+                            working_price = order.order_limit_price
+                    distance = abs(working_price - order.stop_loss)
+                    
+                else:
+                    # Calculate distance using provided pip distance
+                    pip_value = 10**self.get_pip_location(order.instrument)
+                    distance = abs(order.stop_distance * pip_value)
+                
+                # Construct stop loss order details
+                distance = self._check_precision(order.instrument, distance)
+                SL_details = {"distance": str(distance),
+                              "type": "TRAILING_STOP_LOSS"}
             else:
-                stop_loss_details = {"price": str(price)}
+                SL_type = "stopLossOnFill"
+                SL_details = {"price": str(price)}
+            
+            stop_loss_order = {SL_type: SL_details}
+            
         else:
-            stop_loss_details = None
-        
-        return stop_loss_details
+            stop_loss_order = {}
+            
+        return stop_loss_order
     
     
     def _get_take_profit_details(self, order: Order) -> dict:
@@ -686,4 +772,9 @@ class Broker:
         return response.body['positionBook']
     
     
-    
\ No newline at end of file
+    def _get_price(self, instrument: str):
+        """Returns the current price of the instrument.
+        """
+        response = self.api.pricing.get(accountID = self.ACCOUNT_ID, 
+                                        instruments = instrument)
+        return response.body["prices"][0]
\ No newline at end of file
