@@ -20,16 +20,16 @@ class Broker:
         The average spread to use when opening and closing trades.
     margin_available : float
         The margin available on the account.
-    portfolio_balance : float
-        The account balance.
+    equity : float
+        The account equity balance.
     hedging : bool
         Flag whethere hedging is enabled on the account. The default is False.
     home_currency : str
         The default is 'AUD'.
     NAV : float
         The net asset value of the account.
-    unrealised_PL : float
-        The unrealised (floating) PnL.
+    floating_pnl : float
+        The floating PnL.
     verbosity : int
         The verbosity of the broker.
     commission_scheme : str
@@ -49,26 +49,28 @@ class Broker:
         self.trades = {}
         
         # Account 
+        self.NAV = 0 # Net asset value
+        self.equity = 0
+        self.margin_available = 0 
+        
         self.leverage = 1
         self.spread = 0 # TODO - pips or price units? Add docs
-        self.margin_available = 0
-        self.portfolio_balance = 0
         self.hedging = False
         self.margin_closeout = 0.0 # Fraction at margin call
         
-        self.profitable_trades = 0
-        self.peak_value = 0
-        self.low_value = 0
-        self.max_drawdown = 0
         self.home_currency = 'AUD'
-        self.NAV = 0
-        self.unrealised_PL = 0
+        self.floating_pnl = 0
+        
         self.verbosity = broker_config['verbosity']
         
         # Commissions
         self.commission_scheme = 'percentage'
         self.commission = 0
         
+        # History
+        self.account_history = pd.DataFrame()
+        self.holdings = []
+        
     
     def __repr__(self):
         return 'AutoTrader Virtual Broker'
@@ -87,7 +89,7 @@ class Broker:
     def get_balance(self) -> float:
         """Returns balance of account.
         """
-        return self.portfolio_balance
+        return self.equity
     
     
     def place_order(self, order: Order, **kwargs) -> None:
@@ -100,6 +102,11 @@ class Broker:
         
         if order.order_type == 'limit' or order.order_type == 'stop-limit':
             working_price = order.order_limit_price
+        elif order.order_type == 'modify':
+            # Get direction of related trade
+            related_trade = self.trades[order.related_orders]
+            order.direction = related_trade.direction
+            working_price = order.order_price
         else:
             working_price = order.order_price
         
@@ -139,7 +146,7 @@ class Broker:
                 print(f"  Order {order.id} rejected.\n")
             self.cancel_order(order.id, reason)
         else:
-            immediate_orders = ['close', 'reduce']
+            immediate_orders = ['close', 'reduce', 'modify']
             status = 'open' if order.order_type in immediate_orders else 'pending'
             order.status = status
         
@@ -229,19 +236,19 @@ class Broker:
                 total_margin = 0
                 trade_IDs = []
                 
-                for trade_id in open_trades:
-                    trade_IDs.append(trade_id)
-                    total_margin += open_trades[trade_id].margin_required
-                    if open_trades[trade_id].direction > 0:
+                for trade_id, trade in open_trades.items():
+                    trade_IDs.append(trade.id)
+                    total_margin += trade.margin_required
+                    if trade.direction > 0:
                         # Long trade
-                        long_units += open_trades[trade_id].size
-                        long_PL += open_trades[trade_id].unrealised_PL
-                        long_margin += open_trades[trade_id].margin_required
+                        long_units += trade.size
+                        long_PL += trade.unrealised_PL
+                        long_margin += trade.margin_required
                     else:
                         # Short trade
-                        short_units += open_trades[trade_id].size
-                        short_PL += open_trades[trade_id].unrealised_PL
-                        short_margin += open_trades[trade_id].margin_required
+                        short_units += trade.size
+                        short_PL += trade.unrealised_PL
+                        short_margin += trade.margin_required
             
                 # Construct instrument position dict
                 instrument_position = {'long_units': long_units,
@@ -325,8 +332,10 @@ class Broker:
             trade = Trade(order)
             trade.id = trade_id
             trade.fill_price = working_price
+            trade.last_price = working_price
             trade.time_filled = candle.name
             trade.margin_required = margin_required
+            trade.value = position_value
             self.trades[trade_id] = trade
             
             # Subtract spread cost from account NAV
@@ -365,6 +374,7 @@ class Broker:
                 elif order.order_type == 'modify':
                     # Modification order
                     self._modify_order(order)
+                    order.status = 'closed'
                 
                 elif order.order_type == 'close':
                     related_order = order.related_orders
@@ -480,11 +490,23 @@ class Broker:
         self._update_margin(instrument, candle)
         
         # Update unrealised P/L
-        self.unrealised_PL = unrealised_PL
+        self.floating_pnl = unrealised_PL
         
         # Update open position value
-        self.NAV = self.portfolio_balance + self.unrealised_PL
-    
+        self.NAV = self.equity + self.floating_pnl
+        
+        # Update account history
+        account_snapshot = pd.DataFrame(data={'NAV': self.NAV, 
+                                              'equity': self.equity, 
+                                              'margin': self.margin_available,}, 
+                                        index=[candle.name])
+        self.account_history = pd.concat([self.account_history,
+                                          account_snapshot])
+        
+        # TODO - dont record holdings when not plotting, or by specification
+        holdings = self._get_holding_allocations()
+        self.holdings.append(holdings)
+        
     
     def _close_position(self, instrument: str, candle: pd.core.series.Series, 
                         exit_price: float, trade_id=None) -> None:
@@ -534,12 +556,9 @@ class Broker:
         commission = self._calculate_commissions(trade_id, exit_price, size)
         net_profit = gross_PL - commission
         
-        if net_profit > 0:
-            self.profitable_trades += 1
-        
         # Add trade to closed positions
         trade.profit = net_profit
-        trade.balance = self.portfolio_balance
+        trade.balance = self.equity
         trade.exit_price = exit_price
         trade.fees = commission
         if candle is None:
@@ -550,7 +569,6 @@ class Broker:
         
         # Update account
         self._add_funds(net_profit)
-        self._update_MDD()
     
     
     def _reduce_position(self, order: Order) -> None:
@@ -635,18 +653,13 @@ class Broker:
     def _add_funds(self, amount: float) -> None:
         """Adds funds to brokerage account.
         """
-        self.portfolio_balance  += amount
+        self.equity += amount
     
     
     def _make_deposit(self, deposit: float) -> None:
         """Adds deposit to account balance and NAV.
         """
-        if self.portfolio_balance == 0:
-            # If this is the initial deposit, set peak and low values for MDD
-            self.peak_value = deposit
-            self.low_value = deposit
-            
-        self.portfolio_balance += deposit
+        self.equity += deposit
         self.NAV += deposit
         self._update_margin()
     
@@ -674,36 +687,17 @@ class Broker:
                 
                 # Update margin required in trade dict
                 trade.margin_required = margin_required
+                trade.value = position_value
         
         self.margin_available = self.NAV - margin_used
         
         # Check for margin call
-        if self.margin_available/self.NAV < self.margin_closeout:
+        if self.leverage > 1 and self.margin_available/self.NAV < self.margin_closeout:
             # Margin call
             if self.verbosity > 0:
                 print("MARGIN CALL: closing all positions.")
             self._margin_call(instrument, candle)
 
-
-    def _update_MDD(self) -> None:
-        """Function to calculate maximum portfolio drawdown.
-        """
-        balance     = self.portfolio_balance
-        peak_value  = self.peak_value
-        low_value   = self.low_value
-        
-        if balance > peak_value:
-            self.peak_value = balance
-            self.low_value = balance
-            
-        elif balance < low_value:
-            self.low_value = balance
-        
-        MDD = 100*(low_value - peak_value)/peak_value
-        
-        if MDD < self.max_drawdown:
-            self.max_drawdown = MDD
-    
     
     def _modify_order(self, order: Order) -> None:
         """Modify order with updated parameters. Called when order_type = 'modify', 
@@ -748,4 +742,20 @@ class Broker:
         """
         self._close_position(instrument, candle, candle.Close)
     
+    
+    def _get_holding_allocations(self):
+        """Returns a dictionary containing the nominal value of
+        all open trades."""
+        open_trades = self.get_trades()
+        values = {}
+        for trade_id, trade in open_trades.items():
+            if trade.instrument in values:
+                values[trade.instrument] += trade.size * trade.last_price
+            else:
+                values[trade.instrument] = trade.size * trade.last_price
+                
+        if len(values) == 0:
+            values = {None: None}
+            
+        return values
     
\ No newline at end of file
